package org.apache.ojb.odmg;

import org.odmg.Transaction;

/* Copyright 2003-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Offers useful none odmg-standard methods of the odmg {@link org.odmg.Transaction} interface.
 * <p>
 * Note: All listed methods are <strong>not</strong> part of the standard ODMG-api -
 * they are special (proprietary) OJB extensions.
 * </p>
 *
 * @version $Id: TransactionExt.java,v 1.4.2.3 2005/03/23 16:56:29 arminw Exp $
 */
public interface TransactionExt extends Transaction, HasBroker
{
    /**
     * Marks an object for deletion without
     * locking the object. If the object wasn't locked before,
     * OJB will ask for a WRITE lock at commit.
     *
     * @param  anObject Object to be marked
     */
    public void markDelete(Object anObject);

    /**
     * Marks an object as dirty without
     * locking the object. If the object wasn't locked before,
     * OJB will ask for a WRITE lock at commit.
     *
     * @param  anObject Object to be marked
     */
    public void markDirty(Object anObject);

    /**
     * <p>
     * Calling <code>flush</code> flushes persistent object modifications
     * made within the ODMG transaction since the last checkpoint to the underlying
     * database transaction, but does <b<not</b> commit the database transaction.
     * The ODMG transaction retains all locks it held on those objects at the time the flush
     * was invoked.
     * <p/>
     * This method is very similair to {@link org.odmg.Transaction#checkpoint}.
     */
    public void flush();

    /**
     * This method can be used to activate or deactivate the implicit
     * locking mechanism for the current transaction.
     * Turning of implicit locking may improve performance but requires
     * additional care to make sure all changed objects are properly
     * registered to the transaction.
     *
     * @param value If set <em>true</em> implicit locking is enabled,
     *        if <em>false</em>, implicit locking is disabled.
     **/
    public void setImplicitLocking(boolean value);

    /**
     * Returns <em>true</em> if implicite locking is enabled.
     */
    public boolean isImplicitLocking();

    /**
     * Allows to change the <em>cascading delete</em> behavior of the target class's
     * reference field while this transaction is in use.
     *
     * @param target The class to change cascading delete behavior of the references.
     * @param referenceField The field name of the 1:1, 1:n or 1:n reference.
     * @param doCascade If <em>true</em> cascading delete is enabled, <em>false</em> disabled.
     */
    public void setCascadingDelete(Class target, String referenceField, boolean doCascade);

    /**
     * Allows to change the <em>cascading delete</em> behavior of all references of the
     * specified class while this transaction is in use.
     *
     * @param target The class to change cascading delete behavior of all references.
     * @param doCascade If <em>true</em> cascading delete is enabled, <em>false</em> disabled.
     */
    public void setCascadingDelete(Class target, boolean doCascade);
}
