package org.apache.ojb.broker.metadata;

import junit.framework.TestCase;
import org.apache.ojb.broker.PersistenceBroker;
import org.apache.ojb.broker.PBFactoryException;
import org.apache.ojb.broker.PersistenceBrokerFactory;
import org.apache.ojb.broker.PersistenceBrokerException;
import org.apache.ojb.broker.Identity;

/**
 *
 * @author <a href="mailto:armin@codeAuLait.de">Armin Waibel</a>
 * @version $Id: ReadonlyTest.java,v 1.1 2003/12/09 13:46:47 arminw Exp $
 */
public class ReadonlyTest extends TestCase
{
    private PersistenceBroker broker;

    public static void main(String[] args)
    {
        String[] arr = {ReadonlyTest.class.getName()};
        junit.textui.TestRunner.main(arr);
    }

    public void setUp() throws PBFactoryException
    {
        broker = PersistenceBrokerFactory.defaultPersistenceBroker();
    }

    public void tearDown()
    {
        try
        {
            if(broker != null) broker.close();
        }
        catch (PersistenceBrokerException e)
        {
        }
    }

    public void testReadonly() throws Exception
    {
        long timestamp = System.currentTimeMillis();
        TestObject obj = new TestObject(null, "readonly test_"+timestamp, "should not persisted", new Long(timestamp));
        try
        {
            broker.beginTransaction();
            broker.store(obj);
            broker.commitTransaction();
            broker.clearCache();

            broker.beginTransaction();
            Identity oid = new Identity(obj, broker);
            TestObject ret_obj = (TestObject) broker.getObjectByIdentity(oid);
            broker.commitTransaction();

            assertNotNull(ret_obj);
            assertNotNull(ret_obj.getName());
            assertNull("Field should not be populated", ret_obj.getReadonlyLong());
            assertNull("Field should not be populated", ret_obj.getReadonlyString());
        }
        finally
        {
            if(broker != null) broker.close();
        }


    }

    public static class TestObject
    {
        private Long objId;
        private String name;
        private String readonlyString;
        private Long readonlyLong;

        public TestObject()
        {
        }

        public TestObject(Long objId, String name, String readonlyString, Long readonlyLong)
        {
            this.objId = objId;
            this.name = name;
            this.readonlyString = readonlyString;
            this.readonlyLong = readonlyLong;
        }

        public Long getObjId()
        {
            return objId;
        }

        public void setObjId(Long objId)
        {
            this.objId = objId;
        }

        public String getName()
        {
            return name;
        }

        public void setName(String name)
        {
            this.name = name;
        }

        public String getReadonlyString()
        {
            return readonlyString;
        }

        public void setReadonlyString(String readonlyString)
        {
            this.readonlyString = readonlyString;
        }

        public Long getReadonlyLong()
        {
            return readonlyLong;
        }

        public void setReadonlyLong(Long readonlyLong)
        {
            this.readonlyLong = readonlyLong;
        }
    }
}
