package org.apache.ojb.odmg;

import java.util.List;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.apache.ojb.junit.ODMGTestCase;
import org.odmg.OQLQuery;

/* Copyright 2002-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Testing complex object graphs with circular references.
 * 
 * @version $Id: CircularTest.java,v 1.1.2.1 2005/04/03 01:48:10 arminw Exp $
 */
public class CircularTest extends ODMGTestCase
{
    public static void main(String[] args)
    {
        String[] arr = {CircularTest.class.getName()};
        junit.textui.TestRunner.main(arr);
    }

    public void testShowOrderBug_1() throws Exception
    {
        if(ojbSkipKnownIssueProblem("Seems we have a bug in ObjectEnvelopeOrdering")) return;

        String name = "testShowOrderBug_1_" + System.currentTimeMillis();
        Shop s1 = new Shop(name + "_1");
        ShopDetail sd = new ShopDetail(name + "_1");
        s1.setDetail(sd);
        sd.setShop(s1);
        Distributor d1 = new Distributor(name + "_1");
        s1.addDistributor(d1);
        d1.addShop(s1);

        TransactionExt tx = (TransactionExt) odmg.newTransaction();
        tx.begin();
        database.makePersistent(s1);
        tx.commit();
    }

    public void testMtoNWithBackReference_1() throws Exception
    {
        if(ojbSkipKnownIssueProblem("Seems we have a bug in ObjectEnvelopeOrdering")) return;

        String name = "testMtoNWithBackReference_1_" + System.currentTimeMillis();
        Shop s1 = new Shop(name + "_1");
        ShopDetail sd = new ShopDetail(name + "_1");
        s1.setDetail(sd);
        sd.setShop(s1);
        Shop s2 = new Shop(name + "_2");
        Shop s3 = new Shop(name + "_3");
        Distributor d1 = new Distributor(name + "_1");
        Distributor d2 = new Distributor(name + "_2");
        Distributor d3 = new Distributor(name + "_3");
        Distributor d4 = new Distributor(name + "_4");
        s1.addDistributor(d1);
        d1.addShop(s1);
        s1.addDistributor(d2);
        d2.addShop(s1);
        s1.addDistributor(d3);
        d3.addShop(s1);
        s2.addDistributor(d1);
        d1.addShop(s2);
        s3.addDistributor(d4);
        d4.addShop(s3);
        s3.addDistributor(d1);
        d1.addShop(s3);
        d3.addShop(s2);
        s2.addDistributor(d3);
        d3.addShop(s3);
        s3.addDistributor(d3);

        TransactionExt tx = (TransactionExt) odmg.newTransaction();
        tx.begin();
        database.makePersistent(s1);
        database.makePersistent(s2);
        database.makePersistent(s3);
        tx.commit();

        tx.begin();
        database.deletePersistent(d1);
        database.deletePersistent(s1);
        database.deletePersistent(d4);
        tx.commit();

        tx.begin();
        OQLQuery query = odmg.newOQLQuery();
        query.create("select shops from " + Shop.class.getName() + " where name like $1");
        query.bind(name + "%");
        Collection result = (Collection) query.execute();

        assertEquals(2, result.size());

        query = odmg.newOQLQuery();
        query.create("select shops from " + Distributor.class.getName() + " where name like $1");
        query.bind(name + "%");
        result = (Collection) query.execute();

        assertEquals(2, result.size());
        tx.commit();

        tx.begin();
        query = odmg.newOQLQuery();
        query.create("select shops from " + Distributor.class.getName() + " where name like $1");
        query.bind(name + "_3");
        result = (Collection) query.execute();
        tx.commit();

        assertEquals(1, result.size());
        Distributor d3New = (Distributor) result.iterator().next();

        assertNotNull(d3New.getShops());
        assertEquals(2, d3New.getShops().size());
    }

    public void testMtoNWithBackReference_2() throws Exception
    {
        String name = "testMtoNWithBackReference_2_" + System.currentTimeMillis();
        Shop s1 = new Shop(name + "_1");
// we only use the  m:n relation
//        ShopDetail sd = new ShopDetail(name + "_1");
//        s1.setDetail(sd);
//        sd.setShop(s1);
        Shop s2 = new Shop(name + "_2");
        Shop s3 = new Shop(name + "_3");
        Distributor d1 = new Distributor(name + "_1");
        Distributor d2 = new Distributor(name + "_2");
        Distributor d3 = new Distributor(name + "_3");
        Distributor d4 = new Distributor(name + "_4");
        s1.addDistributor(d1);
        d1.addShop(s1);
        s1.addDistributor(d2);
        d2.addShop(s1);
        s1.addDistributor(d3);
        d3.addShop(s1);
        s2.addDistributor(d1);
        d1.addShop(s2);
        s3.addDistributor(d4);
        d4.addShop(s3);
        s3.addDistributor(d1);
        d1.addShop(s3);
        d3.addShop(s2);
        s2.addDistributor(d3);
        d3.addShop(s3);
        s3.addDistributor(d3);

        TransactionExt tx = (TransactionExt) odmg.newTransaction();
        tx.begin();
        database.makePersistent(s1);
        database.makePersistent(s2);
        database.makePersistent(s3);
        tx.commit();

        tx.begin();
        tx.setCascadingDelete(Distributor.class, "shops", true);
        database.deletePersistent(d1);
        database.deletePersistent(s1);
        database.deletePersistent(d4);
        tx.commit();

        tx.begin();
        OQLQuery query = odmg.newOQLQuery();
        query.create("select shops from " + Shop.class.getName() + " where name like $1");
        query.bind(name + "%");
        Collection result = (Collection) query.execute();

        assertEquals(0, result.size());

        query = odmg.newOQLQuery();
        query.create("select shops from " + Distributor.class.getName() + " where name like $1");
        query.bind(name + "%");
        result = (Collection) query.execute();

        assertEquals(2, result.size());
        tx.commit();

        tx.begin();
        query = odmg.newOQLQuery();
        query.create("select shops from " + Distributor.class.getName() + " where name like $1");
        query.bind(name + "_3");
        result = (Collection) query.execute();
        tx.commit();

        assertEquals(1, result.size());
        Distributor d3New = (Distributor) result.iterator().next();

        assertNotNull(d3New.getShops());
        assertEquals(0, d3New.getShops().size());
    }

    public void testOneToNWithBackReference_1() throws Exception
    {
        String name = "testOneToNWithBackReference_1_" + System.currentTimeMillis();
        Shop s = new Shop();
        s.setName(name);
        Product p1 = new Product();
        p1.setName(name + "_product_1");
        Product p2 = new Product();
        p2.setName(name + "_product_2");

        List products = new ArrayList();
        products.add(p1);
        products.add(p2);

        s.setProducts(products);
        p1.setShop(s);
        p2.setShop(s);

        Product p1a = new Product();
        p1a.setName(name + "_subProduct_A");
        Product p1b = new Product();
        p1b.setName(name + "_subProduct_B");
        Product p1c = new Product();
        p1c.setName(name + "_subProduct_C");

        List subProducts = new ArrayList();
        subProducts.add(p1a);
        subProducts.add(p1b);
        subProducts.add(p1c);

        p1.setSubProducts(subProducts);

        TransactionExt tx = (TransactionExt) odmg.newTransaction();
        tx.begin();
        database.makePersistent(s);
        tx.commit();

        tx.begin();
        tx.getBroker().clearCache();

        OQLQuery query = odmg.newOQLQuery();
        query.create("select products from " + Shop.class.getName() + " where name like $1");
        query.bind(name + "%");
        Collection result = (Collection) query.execute();
        tx.commit();

        assertEquals(1, result.size());
        Shop newShop = (Shop) result.iterator().next();
        assertNotNull(newShop.getProducts());
        assertEquals(2, newShop.getProducts().size());

        boolean match = false;
        for(Iterator iterator = newShop.getProducts().iterator(); iterator.hasNext();)
        {
            Product p =  (Product) iterator.next();
            if(p.getSubProducts() != null && p.getSubProducts().size() > 0)
            {
                match = true;
                assertEquals(3, p.getSubProducts().size());
            }
        }
        assertTrue("Sub products aren't stored", match);

        tx.begin();
        database.deletePersistent(s);
        tx.commit();


        tx.begin();
        tx.getBroker().clearCache();
        query = odmg.newOQLQuery();
        query.create("select products from " + Product.class.getName() + " where name like $1");
        query.bind(name + "_subPro%");
        result = (Collection) query.execute();
        tx.commit();
        assertEquals(3, result.size());

        tx.begin();
        query = odmg.newOQLQuery();
        query.create("select products from " + Product.class.getName() + " where name like $1");
        query.bind(name + "_product_1");
        result = (Collection) query.execute();
        assertEquals(1, result.size());

        database.deletePersistent(result.iterator().next());
        tx.commit();


        tx.begin();
        tx.getBroker().clearCache();
        query = odmg.newOQLQuery();
        query.create("select products from " + Product.class.getName() + " where name like $1");
        query.bind(name + "_subPro%");
        result = (Collection) query.execute();
        tx.commit();
        assertEquals(3, result.size());
    }

    public void testOneToNWithBackReference_2() throws Exception
    {
        String name = "testOneToNWithBackReference_2_" + System.currentTimeMillis();
        Shop s = new Shop();
        s.setName(name);
        Product p1 = new Product();
        p1.setName(name + "_product_1");
        Product p2 = new Product();
        p2.setName(name + "_product_2");

        List products = new ArrayList();
        products.add(p1);
        products.add(p2);

        s.setProducts(products);
        p1.setShop(s);
        p2.setShop(s);

        Product p1a = new Product();
        p1a.setName(name + "_subProduct_A");
        Product p1b = new Product();
        p1b.setName(name + "_subProduct_B");
        Product p1c = new Product();
        p1c.setName(name + "_subProduct_C");

        List subProducts = new ArrayList();
        subProducts.add(p1a);
        subProducts.add(p1b);
        subProducts.add(p1c);

        p1.setSubProducts(subProducts);

        TransactionExt tx = (TransactionExt) odmg.newTransaction();
        tx.begin();
        database.makePersistent(s);
        tx.commit();

        tx.begin();
        tx.getBroker().clearCache();

        OQLQuery query = odmg.newOQLQuery();
        query.create("select products from " + Shop.class.getName() + " where name like $1");
        query.bind(name + "%");
        Collection result = (Collection) query.execute();
        tx.commit();

        assertEquals(1, result.size());
        Shop newShop = (Shop) result.iterator().next();
        assertNotNull(newShop.getProducts());
        assertEquals(2, newShop.getProducts().size());

        boolean match = false;
        for(Iterator iterator = newShop.getProducts().iterator(); iterator.hasNext();)
        {
            Product p =  (Product) iterator.next();
            if(p.getSubProducts() != null && p.getSubProducts().size() > 0)
            {
                match = true;
                assertEquals(3, p.getSubProducts().size());
            }
        }
        assertTrue("Sub products aren't stored", match);

        tx.begin();
        // now we enable cascading delete
        tx.setCascadingDelete(Shop.class, true);
        database.deletePersistent(s);
        tx.commit();


        tx.begin();
        tx.getBroker().clearCache();
        query = odmg.newOQLQuery();
        query.create("select products from " + Product.class.getName() + " where name like $1");
        query.bind(name + "_subPro%");
        result = (Collection) query.execute();
        assertEquals(3, result.size());

        query = odmg.newOQLQuery();
        query.create("select products from " + Product.class.getName() + " where name like $1");
        query.bind(name + "_product_1");
        result = (Collection) query.execute();
        assertEquals(0, result.size());

        tx.commit();
    }

    public void testOneToNWithBackReference_3() throws Exception
    {
        String name = "testOneToNWithBackReference_3_" + System.currentTimeMillis();
        Shop s = new Shop();
        s.setName(name);
        Product p1 = new Product();
        p1.setName(name + "_product_1");
        Product p2 = new Product();
        p2.setName(name + "_product_2");

        List products = new ArrayList();
        products.add(p1);
        products.add(p2);

        s.setProducts(products);
        p1.setShop(s);
        p2.setShop(s);

        Product p1a = new Product();
        p1a.setName(name + "_subProduct_A");
        Product p1b = new Product();
        p1b.setName(name + "_subProduct_B");
        Product p1c = new Product();
        p1c.setName(name + "_subProduct_C");

        List subProducts = new ArrayList();
        subProducts.add(p1a);
        subProducts.add(p1b);
        subProducts.add(p1c);

        p1.setSubProducts(subProducts);

        TransactionExt tx = (TransactionExt) odmg.newTransaction();
        tx.begin();
        database.makePersistent(s);
        tx.commit();

        tx.begin();
        tx.getBroker().clearCache();

        OQLQuery query = odmg.newOQLQuery();
        query.create("select products from " + Shop.class.getName() + " where name like $1");
        query.bind(name + "%");
        Collection result = (Collection) query.execute();
        tx.commit();

        assertEquals(1, result.size());
        Shop newShop = (Shop) result.iterator().next();
        assertNotNull(newShop.getProducts());
        assertEquals(2, newShop.getProducts().size());

        boolean match = false;
        for(Iterator iterator = newShop.getProducts().iterator(); iterator.hasNext();)
        {
            Product p =  (Product) iterator.next();
            if(p.getSubProducts() != null && p.getSubProducts().size() > 0)
            {
                match = true;
                assertEquals(3, p.getSubProducts().size());
            }
        }
        assertTrue("Sub products aren't stored", match);

        tx.begin();
        // now we enable cascading delete
        tx.setCascadingDelete(Shop.class, true);
        tx.setCascadingDelete(Product.class, true);
        database.deletePersistent(s);
        tx.commit();


        tx.begin();
        tx.getBroker().clearCache();
        query = odmg.newOQLQuery();
        query.create("select products from " + Product.class.getName() + " where name like $1");
        query.bind(name + "_subPro%");
        result = (Collection) query.execute();
        assertEquals(0, result.size());

        query = odmg.newOQLQuery();
        query.create("select products from " + Product.class.getName() + " where name like $1");
        query.bind(name + "_product_1");
        result = (Collection) query.execute();
        assertEquals(0, result.size());

        tx.commit();
    }

    public void testOneToOneWithBackReference_1() throws Exception
    {
        String name = "testOneToOneWithBackReference_1_" + System.currentTimeMillis();
        Shop s = new Shop();
        s.setName(name);
        ShopDetail sd = new ShopDetail();
        sd.setName(name);
        s.setDetail(sd);
        sd.setShop(s);

        TransactionExt tx = (TransactionExt) odmg.newTransaction();
        tx.begin();
        database.makePersistent(s);
        tx.commit();

        tx.begin();
        tx.getBroker().clearCache();

        OQLQuery query = odmg.newOQLQuery();
        query.create("select detail from " + ShopDetail.class.getName() + " where name like $1");
        query.bind(name);
        Collection result = (Collection) query.execute();
        tx.commit();

        assertEquals(1, result.size());
        ShopDetail sdNew = (ShopDetail) result.iterator().next();
        assertNotNull(sdNew.getShop());

        tx.begin();
        database.deletePersistent(sdNew);
        tx.flush();

        query = odmg.newOQLQuery();
        query.create("select detail from " + ShopDetail.class.getName() + " where name like $1");
        query.bind(name);
        result = (Collection) query.execute();
        tx.commit();
        assertEquals(0, result.size());

        tx.begin();
        query = odmg.newOQLQuery();
        query.create("select detail from " + Shop.class.getName() + " where name like $1");
        query.bind(name);
        result = (Collection) query.execute();

        assertEquals(1, result.size());

        database.deletePersistent(result.iterator().next());
        tx.commit();
    }

    public void testOneToOneWithBackReference_3() throws Exception
    {
        if(ojbSkipKnownIssueProblem("Seems we have a bug in ObjectEnvelopeOrdering")) return;

        String name = "testOneToOneWithBackReference_3_" + System.currentTimeMillis();
        Shop s = new Shop();
        s.setName(name);
        ShopDetail sd = new ShopDetail();
        sd.setName(name);
        s.setDetail(sd);
        sd.setShop(s);
        TransactionExt tx = (TransactionExt) odmg.newTransaction();
        tx.begin();
        database.makePersistent(sd);
        database.makePersistent(s);
        database.deletePersistent(s);
        database.makePersistent(sd);
        database.makePersistent(s);
        tx.commit();

        tx.begin();
        OQLQuery query = odmg.newOQLQuery();
        query.create("select detail from " + ShopDetail.class.getName() + " where name like $1");
        query.bind(name);
        Collection result = (Collection) query.execute();
        tx.commit();

        assertEquals(1, result.size());
        ShopDetail sdNew = (ShopDetail) result.iterator().next();
        assertNotNull(sdNew.getShop());

        tx.begin();
        tx.setCascadingDelete(ShopDetail.class, true);
        database.deletePersistent(sdNew);
        database.makePersistent(sdNew);
        tx.flush();

        query = odmg.newOQLQuery();
        query.create("select detail from " + ShopDetail.class.getName() + " where name like $1");
        query.bind(name);
        result = (Collection) query.execute();
        tx.commit();
        assertEquals(1, result.size());

        tx.begin();
        database.deletePersistent(sdNew);
        database.makePersistent(sdNew);
        database.deletePersistent(sdNew);
        tx.flush();

        query = odmg.newOQLQuery();
        query.create("select detail from " + Shop.class.getName() + " where name like $1");
        query.bind(name);
        result = (Collection) query.execute();
        assertEquals(0, result.size());
        tx.commit();
    }

    public void testOneToOneWithBackReference_2() throws Exception
    {
        String name = "testOneToOneWithBackReference_2_" + System.currentTimeMillis();
        Shop s = new Shop();
        s.setName(name);
        ShopDetail sd = new ShopDetail();
        sd.setName(name);
        s.setDetail(sd);
        sd.setShop(s);

        TransactionExt tx = (TransactionExt) odmg.newTransaction();
        tx.begin();
        database.makePersistent(s);
        tx.commit();

        tx.begin();
        tx.getBroker().clearCache();

        OQLQuery query = odmg.newOQLQuery();
        query.create("select detail from " + ShopDetail.class.getName() + " where name like $1");
        query.bind(name);
        Collection result = (Collection) query.execute();
        tx.commit();

        assertEquals(1, result.size());
        ShopDetail sdNew = (ShopDetail) result.iterator().next();
        assertNotNull(sdNew.getShop());

        tx.begin();
        tx.setCascadingDelete(ShopDetail.class, true);
        database.deletePersistent(sdNew);
        tx.flush();

        query = odmg.newOQLQuery();
        query.create("select detail from " + ShopDetail.class.getName() + " where name like $1");
        query.bind(name);
        result = (Collection) query.execute();
        tx.commit();
        assertEquals(0, result.size());

        tx.begin();
        query = odmg.newOQLQuery();
        query.create("select detail from " + Shop.class.getName() + " where name like $1");
        query.bind(name);
        result = (Collection) query.execute();
        assertEquals(0, result.size());
        tx.commit();
    }

    public static class Shop
    {
        private Integer id;
        private String name;
        private ShopDetail detail;
        private List products;
        private List distributors;

        public Shop()
        {
        }

        public Shop(String name)
        {
            this.name = name;
        }

        public Integer getId()
        {
            return id;
        }

        public void setId(Integer id)
        {
            this.id = id;
        }

        public String getName()
        {
            return name;
        }

        public void setName(String name)
        {
            this.name = name;
        }

        public ShopDetail getDetail()
        {
            return detail;
        }

        public void setDetail(ShopDetail detail)
        {
            this.detail = detail;
        }

        public List getProducts()
        {
            return products;
        }

        public void setProducts(List products)
        {
            this.products = products;
        }

        public List getDistributors()
        {
            return distributors;
        }

        public void setDistributors(List distributors)
        {
            this.distributors = distributors;
        }

        public void addDistributor(Distributor d)
        {
            if(this.distributors == null)
            {
                this.distributors = new ArrayList();
            }
            this.distributors.add(d);
        }
    }

    public static class Distributor
    {
        private Integer id;
        private String name;
        private List shops;

        public Distributor()
        {
        }

        public Distributor(String name)
        {
            this.name = name;
        }

        public Integer getId()
        {
            return id;
        }

        public void setId(Integer id)
        {
            this.id = id;
        }

        public String getName()
        {
            return name;
        }

        public void setName(String name)
        {
            this.name = name;
        }

        public List getShops()
        {
            return shops;
        }

        public void setShops(List shops)
        {
            this.shops = shops;
        }

        public void addShop(Shop s)
        {
            if(this.shops == null)
            {
                this.shops = new ArrayList();
            }
            this.shops.add(s);
        }
    }

    public static class Product
    {
        private Integer id;
        private String name;
        private Shop shop;
        private Integer shopFk;
        private List subProducts;
        private Integer subProductFK;

        public Product()
        {
        }

        public Integer getId()
        {
            return id;
        }

        public void setId(Integer id)
        {
            this.id = id;
        }

        public String getName()
        {
            return name;
        }

        public void setName(String name)
        {
            this.name = name;
        }

        public Shop getShop()
        {
            return shop;
        }

        public void setShop(Shop shop)
        {
            this.shop = shop;
        }

        public Integer getShopFk()
        {
            return shopFk;
        }

        public void setShopFk(Integer shopFk)
        {
            this.shopFk = shopFk;
        }

        public Integer getSubProductFK()
        {
            return subProductFK;
        }

        public void setSubProductFK(Integer subProductFK)
        {
            this.subProductFK = subProductFK;
        }

        public List getSubProducts()
        {
            return subProducts;
        }

        public void setSubProducts(List subProducts)
        {
            this.subProducts = subProducts;
        }
    }

    public static class ShopDetail
    {
        private Integer id;
        private String name;
        private Shop shop;

        public ShopDetail()
        {
        }

        public ShopDetail(String name)
        {
            this.name = name;
        }

        public Integer getId()
        {
            return id;
        }

        public void setId(Integer id)
        {
            this.id = id;
        }

        public String getName()
        {
            return name;
        }

        public void setName(String name)
        {
            this.name = name;
        }

        public Shop getShop()
        {
            return shop;
        }

        public void setShop(Shop shop)
        {
            this.shop = shop;
        }
    }
}
